@@ -92,8 +92,9 @@ type merlin = {
   flags_applied : string list with_workdir list;
 
   failures : string list;
-  extension_to_reader : (string * string) list
+  extension_to_reader : (string * string) list;
 
+  cache_lifespan : int
 }
 
 let dump_merlin x =
@@ -129,7 +130,8 @@ let dump_merlin x =
           "extension", `String suffix;
           "reader", `String reader;
         ]) x.extension_to_reader
-    )
+    );
+    "cache_lifespan"   , Json.string (string_of_int x.cache_lifespan)
   ]
 
 module Verbosity = struct
@@ -230,6 +232,26 @@ let rec normalize t =
   ) else
     normalize (normalize_step t)
 
+let merge_merlin_config dot merlin ~failures ~config_path =
+  { merlin with
+    build_path = dot.Mconfig_dot.build_path @ merlin.build_path;
+    source_path = dot.source_path @ merlin.source_path;
+    cmi_path = dot.cmi_path @ merlin.cmi_path;
+    cmt_path = dot.cmt_path @ merlin.cmt_path;
+    exclude_query_dir = dot.exclude_query_dir || merlin.exclude_query_dir;
+    use_ppx_cache = dot.use_ppx_cache || merlin.use_ppx_cache;
+    extensions = dot.extensions @ merlin.extensions;
+    suffixes = dot.suffixes @ merlin.suffixes;
+    stdlib = (if dot.stdlib = None then merlin.stdlib else dot.stdlib);
+    reader =
+      if dot.reader = []
+      then merlin.reader
+      else dot.reader;
+    flags_to_apply = dot.flags @ merlin.flags_to_apply;
+    failures = failures @ merlin.failures;
+    config_path = Some config_path;
+  }
+
 let get_external_config path t =
   let path = Misc.canonicalize_filename path in
   let directory = Filename.dirname path in
@@ -237,26 +259,7 @@ let get_external_config path t =
   | None -> t
   | Some (ctxt, config_path) ->
     let dot, failures = Mconfig_dot.get_config ctxt path in
-    let merlin = t.merlin in
-    let merlin = {
-      merlin with
-      build_path = dot.build_path @ merlin.build_path;
-      source_path = dot.source_path @ merlin.source_path;
-      cmi_path = dot.cmi_path @ merlin.cmi_path;
-      cmt_path = dot.cmt_path @ merlin.cmt_path;
-      exclude_query_dir = dot.exclude_query_dir || merlin.exclude_query_dir;
-      use_ppx_cache = dot.use_ppx_cache || merlin.use_ppx_cache;
-      extensions = dot.extensions @ merlin.extensions;
-      suffixes = dot.suffixes @ merlin.suffixes;
-      stdlib = (if dot.stdlib = None then merlin.stdlib else dot.stdlib);
-      reader =
-        if dot.reader = []
-        then merlin.reader
-        else dot.reader;
-      flags_to_apply = dot.flags @ merlin.flags_to_apply;
-      failures = failures @ merlin.failures;
-      config_path = Some config_path;
-    } in
+    let merlin = merge_merlin_config dot t.merlin ~failures ~config_path in
     normalize { t with merlin }
 
 let merlin_flags = [
@@ -358,6 +361,15 @@ let merlin_flags = [
     marg_path (fun path merlin -> {merlin with stdlib = Some path}),
     "<path> Change path of ocaml standard library"
   );
+  (
+    "-cache-lifespan",
+    Marg.param "int" (fun prot merlin ->
+        try {merlin with cache_lifespan = (int_of_string prot)}
+        with _ -> invalid_arg "Valid value is int";
+      ),
+    "Change file cache retention period. It's measured in minutes. \
+      Default value is 5."
+  );
   (
     (* Legacy support for janestreet. Ignored. To be removed soon. *)
     "-attributes-allowed",
@@ -627,6 +639,7 @@ let initial = {
 
     failures = [];
     extension_to_reader = [(".re","reason");(".rei","reason")];
+    cache_lifespan = 5;
   };
   query = {
     filename = "*buffer*";
